@@ -28,6 +28,7 @@
 #include <qpushbutton.h>
 #include <qstylesheet.h>
 #include <ktextbrowser.h>
+#include <klistview.h>
 
 class Prefix;
 class QalculatePrecisionDialog;
@@ -50,7 +51,7 @@ class KAction;
 class KRadioAction;
 class KToggleAction;
 class QToolBar;
-class KPopupMenu;
+class QPopupMenu;
 class KPushButton;
 class QalculateExpressionEdit;
 class QalculateResultDisplay;
@@ -62,8 +63,7 @@ class QalculateHistoryBrowser;
 class QalculateConvertNumberBasesDialog;
 class QalculatePeriodicTableDialog;
 class QalculatePlotDialog;
-class QalculateSetBaseInExpressionDialog;
-class QalculateSetBaseOtherDialog;
+class QalculateSetBaseDialog;
 class KLineEdit;
 class MathStructure;
 class QalculateImportCSVDialog;
@@ -77,6 +77,10 @@ class QRadioButton;
 class QalculateButton;
 class QVBoxLayout;
 class KSeparator;
+class KUniqueApplication;
+class KConfig;
+class QListViewItem;
+class QalculateRPNStackListView;
 
 void *view_proc(void *pipe);
 void *command_proc(void *pipe);
@@ -91,14 +95,15 @@ class KQalculate : public KMainWindow {
 	Q_OBJECT
 
 public:
-	KQalculate( QWidget* parent = 0, const char* name = 0, WFlags fl = WType_TopLevel | WDestructiveClose);
+
+	KQalculate(KUniqueApplication *parent = 0, const char* name = 0, WFlags fl = WType_TopLevel | WDestructiveClose);
 	virtual ~KQalculate();
 
 	void setupActions();
 	void setAssumptionsMenu();
 	void setModeActions();
 	void execute_expression(bool force = true);
-	void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, QString transformation = "");
+	void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, QString transformation = "", size_t stack_index = 0, bool register_moved = false);
 	void executeCommand(int command_type);
 	void result_display_updated();
 	void result_format_updated();
@@ -128,6 +133,13 @@ public:
 	const MathStructure *getResultPart(int);
 	void insertMatrixVector(const MathStructure *m = NULL, bool do_vector = false, bool is_text_struct = false, bool is_result = false);
 
+	void calculateRPN(int op);
+	void calculateRPN(MathFunction *f);
+	void RPNStackEnter(Variable *v);
+	void RPNRegisterAdded(const QString &text, size_t index = 0);
+	void RPNRegisterRemoved(size_t index);
+	void RPNRegisterChanged(const QString &text, size_t index = 0);
+
 	QalculateInsertMatrixVectorDialog *insert_matrix_dialog;
 	QalculatePrecisionDialog *precisionDialog;
 	QalculateDecimalsDialog *decimalsDialog;
@@ -147,8 +159,7 @@ public:
 	QalculateImportCSVDialog *import_csv_dialog;
 	QalculateExportCSVDialog *export_csv_dialog;
 	QalculatePlotDialog *plot_dialog;
-	QalculateSetBaseInExpressionDialog *set_base_in_expression_dialog;
-	QalculateSetBaseOtherDialog *set_base_other_dialog;
+	QalculateSetBaseDialog *set_base_dialog;
 	QalculatePreferencesDialog *preferences_dialog;
 	QMap <int, Variable*> menu_variables_ids;
 	QMap <int, MathFunction*> menu_functions_ids;
@@ -158,6 +169,11 @@ public:
 	QValueVector<int> recent_variable_ids;
 	QValueVector<int> recent_unit_ids;
 
+	QalculateRPNStackListView *stackList;
+	QVBoxLayout *stackPageButtonsLayout;
+	QalculateButton *registerUpButton, *registerDownButton, *deleteRegisterButton, *editRegisterButton, *clearStackButton;
+	QPopupMenu *stackMenu;
+	KUniqueApplication *m_parent;
 	KSystemTray *trayicon;
 	QalculateButton *executeButton, *storeButton, *convertButton;
 	QalculateButton *functionsButton;
@@ -169,7 +185,7 @@ public:
 	QalculateResultDisplay* resultLabel;
 	QWidgetStack* mainStack;
 	QFrame *bottomLine;
-	QWidget *keypadPage, *historyPage;
+	QWidget *keypadPage, *historyPage, *stackPage;
 	QComboBox* kpNumericCombo, *kpBaseCombo;
 	QRadioButton *kpDegreesButton, *kpRadiansButton, *kpGradiansButton, *kpNoAngleUnitButton;
 	QButtonGroup *kpAngleGroup;
@@ -178,11 +194,11 @@ public:
 	QalculateButton *kpTan, *kpLn, *kpFactorial, *kpMod, *kpAns, *kp8, *kp0, *kp7, *kpDel, *kpDot, *kp5, *kpEquals, *kpClear;
 	QalculateButton *kp3, *kp1, *kp4, *kp9, *kpDivision, *kpExp, *kp6, *kp2, *kpMinus, *kpTimes, *kpPlus;
 	QalculateHistoryBrowser *historyBrowser;
-	QPushButton *historyButton, *keypadButton;
+	QPushButton *historyButton, *keypadButton, *stackButton;
 	KMenuBar *menubar;
 	QPopupMenu *menu_functions, *menu_variables, *menu_units, *menu_to_unit, *menu_set_prefix, *menu_modes;
 	KToggleAction *ActionAbbreviateNames, *ActionReadPrecision, *ActionNonZeroDenominators, *ActionWarnAboutDenominatorsAssumedNonZero;
-	KToggleAction *ActionLimitImplicitMultiplication, *ActionRPNMode, *ActionIndicateInfiniteSeries;
+	KToggleAction *ActionLimitImplicitMultiplication, *ActionRPNMode, *ActionRPNSyntax, *ActionIndicateInfiniteSeries;
 	KToggleAction *ActionShowEndingZeroes, *ActionRoundHalfwayNumbersToEven, *ActionEnablePrefixes;
 	KToggleAction *ActionEnableUseOfAllPrefixes, *ActionEnableDenominatorPrefixes, *ActionPlaceUnitsSeparately;
 	KToggleAction *ActionEnableVariables, *ActionEnableFunctions, *ActionEnableUnits, *ActionEnableUnknowns;
@@ -192,14 +208,14 @@ public:
 	KRadioAction *ActionFractionalDisplayDecimal, *ActionFractionalDisplayDecimalTryExact;
 	KRadioAction *ActionFractionalDisplayFraction, *ActionFractionalDisplayCombined, *ActionNumericalDisplayNormal;
 	KRadioAction *ActionNumericalDisplayScientific, *ActionNumericalDisplayPurelyScientific;
-	KRadioAction *ActionNumericalDisplaySimple, *ActionNumberBaseBinary, *ActionNumberBaseOctal, *ActionNumberBaseDecimal, *ActionNumberBaseHexadecimal;
+	KRadioAction *ActionNumericalDisplaySimple, *ActionNumberBaseBinary, *ActionNumberBaseOctal, *ActionNumberBaseDecimal, *ActionNumberBaseHexadecimal, *ActionNumberBaseOther;
 	KRadioAction *ActionNumberBaseSexagesimal, *ActionNumberBaseTimeFormat, *ActionNumberBaseRomanNumerals;
 	KRadioAction *ActionAssumptionTypeUnknown, *ActionAssumptionTypeNonMatrix, *ActionAssumptionTypeNumber, *ActionAssumptionTypeComplex, *ActionAssumptionTypeReal;
 	KRadioAction *ActionAssumptionTypeRational, *ActionAssumptionTypeInteger, *ActionAssumptionSignUnknown, *ActionAssumptionSignNonZero;
 	KRadioAction *ActionAssumptionSignPositive, *ActionAssumptionSignNonNegative, *ActionAssumptionSignNegative;
 	KRadioAction *ActionAssumptionSignNonPositive, *ActionAngleUnitDegrees, *ActionAngleUnitRadians, *ActionAngleUnitGradians;
 	KRadioAction *ActionAngleUnitNone, *ActionAutoNoConversion, *ActionAutoConvertToBaseUnits, *ActionAutoConvertToBestUnit;
-	KAction *ActionNumberBaseOther, *ActionSetBaseInExpression, *ActionPrecision, *ActionDecimals, *ActionSaveModeAs, *ActionDeleteMode, *ActionSaveMode, *ActionQuit;
+	KAction *ActionSetBase, *ActionPrecision, *ActionDecimals, *ActionSaveModeAs, *ActionDeleteMode, *ActionSaveMode, *ActionQuit;
 	KAction *ActionUpdateExchangeRates, *ActionManageVariables, *ActionConvertToUnitExpression, *ActionStoreResult, *ActionClose;
 	KAction *ActionConvertNumberBases, *ActionManageFunctions, *ActionConvertToBaseUnits, *ActionConvertToBestUnit;
 	KAction *ActionManageUnits, *ActionManageDataSets, *ActionFactorize, *ActionSimplify, *ActionSetUnknowns, *ActionCopyResult;
@@ -208,6 +224,8 @@ public:
 	KAction *ActionNewUnknownVariable, *ActionNewFunction, *ActionNewUnit, *ActionNewDataSet;
 	KAction *ActionInsertMatrix, *ActionInsertVector;
 	KAction *ActionClearHistory;
+	KAction *ActionHideSystemTrayIcon, *ActionHide;
+	KAction *ActionClearStack, *ActionDeleteRegister, *ActionEditRegister;
 
 public slots:
 
@@ -220,6 +238,7 @@ public slots:
 	void update_umenus();
 	void insertVariable(Variable*);
 	void insertFunction(MathFunction*, QWidget *parent);
+	void applyFunction(MathFunction*, QWidget *parent);
 	void onExpressionChanged();
 	void onErrorTimeout();
 	bool fetch_exchange_rates(int);
@@ -235,6 +254,7 @@ public slots:
 	void setMinDecimals(int);
 	void execute();
 	void toggleHistory(bool);
+	void toggleStack(bool);
 	void toggleKeypad(bool);
 	void showEndingZeroes(bool);
 	void approximationTryExact();
@@ -255,15 +275,16 @@ public slots:
 	void numberBaseBinary();
 	void numberBaseOctal();
 	void numberBaseDecimal();
-	void setOtherNumberBase(int);
 	void numberBaseOther();
 	void numberBaseHexadecimal();
 	void numberBaseSexagesimal();
 	void numberBaseTimeFormat();
 	void numberBaseRomanNumerals();
+	void setBaseInResultFromDialogGroup(int);
+	void setBaseInResultFromDialogBox(int);
 	void setBaseInExpressionFromDialogGroup(int);
 	void setBaseInExpressionFromDialogBox(int);
-	void setBaseInExpression();
+	void setBase();
 	void nonZeroDenominators(bool);
 	void warnAboutDenominatorsAssumedNonZero(bool);
 	void algebraicModeSimplify();
@@ -275,6 +296,7 @@ public slots:
 	void precision();
 	void decimals();
 	void rpnMode(bool);
+	void rpnSyntax(bool);
 	void assumptionSignUnknown();
 	void assumptionSignNonZero();
 	void assumptionSignPositive();
@@ -353,6 +375,7 @@ public slots:
 	void convertToUnitExpression();
 	void convertNumberBases();
 	void insertManagedFunction(MathFunction*);
+	void applyManagedFunction(MathFunction*);
 	void manageFunctions();
 	void manageUnits();
 	void insertUnit(Unit *u);
@@ -386,10 +409,23 @@ public slots:
 	void insertMatrix();
 	void insertVector();
 	void showSystemTrayIcon(bool);
+	void hideSystemTrayIcon();
 	void displayParseStatus();
+	void registerUp();
+	void registerDown();
+	void clearStack();
+	void deleteRegister();
+	void editRegister();
+	void updateRPNIndexes();
+	QListViewItem *getRPNRegister(size_t index);
+	void stackRegisterMoved();
+	void stackRegisterSet(QListViewItem*);
+	void popupStackMenu(KListView*, QListViewItem*, const QPoint&);
+	void registerSelected();
 
 protected:
 
+	void updateButtonWidths();
 	void applySetUnknowns(bool okclicked);
 	bool b_unknowns_changed;
 	QValueVector<bool> unknown_changed;
@@ -399,6 +435,8 @@ protected:
 	QString parsed_expression;
 	void keyPressEvent(QKeyEvent *e);
 	void fontChange(const QFont &old_font);
+	void saveProperties(KConfig *cfg);
+	void readProperties(KConfig *cfg);
 	
 private:
 
@@ -476,5 +514,19 @@ public:
 
 };
 
+class QalculateRPNStackListView : public KListView {
+
+	Q_OBJECT
+
+public:
+
+	QalculateRPNStackListView(QWidget *parent = 0, const char *name = 0);
+	virtual ~QalculateRPNStackListView();
+
+protected:
+
+	void keyPressEvent(QKeyEvent *e);
+
+};
 
 #endif // QALCULATE_KWINDOW_H
