@@ -39,6 +39,8 @@
 
 PrintOptions printops;
 EvaluationOptions evalops;
+bool rpn_mode;
+bool rpn_keypad_only;
 bool load_global_defs, fetch_exchange_rates_at_startup, first_time, first_qalculate_run, save_mode_on_exit, save_defs_on_exit;
 bool use_custom_result_font, use_custom_expression_font, use_custom_status_font;
 int use_icon_buttons;
@@ -80,7 +82,7 @@ extern QWidget *expressionWidget;
 extern KQalculate *mainWin;
 QStringList expression_history;
 
-bool show_keypad, show_history;
+bool show_keypad, show_history, show_stack;
 
 vector<mode_struct> modes;
 
@@ -101,6 +103,7 @@ void set_saved_mode() {
 	modes[1].eo = evalops;
 	modes[1].at = CALCULATOR->defaultAssumptions()->type();
 	modes[1].as = CALCULATOR->defaultAssumptions()->sign();
+	modes[1].rpn_mode = rpn_mode;
 }
 
 size_t save_mode_as(QString name, bool *new_mode) {
@@ -124,6 +127,7 @@ size_t save_mode_as(QString name, bool *new_mode) {
 	modes[index].at = CALCULATOR->defaultAssumptions()->type();
 	modes[index].as = CALCULATOR->defaultAssumptions()->sign();
 	modes[index].name = name;
+	modes[index].rpn_mode = rpn_mode;
 	return index;
 }
 
@@ -171,6 +175,7 @@ void load_preferences() {
 	printops.allow_non_usable = false;
 	printops.lower_case_numbers = false;
 	printops.lower_case_e = false;
+	printops.base_display = BASE_DISPLAY_NORMAL;
 	printops.division_sign = DIVISION_SIGN_DIVISION_SLASH;
 	printops.multiplication_sign = MULTIPLICATION_SIGN_DOT;
 	printops.can_display_unicode_string_function = &can_display_unicode_string_function;
@@ -190,6 +195,10 @@ void load_preferences() {
 	evalops.assume_denominators_nonzero = true;
 	evalops.warn_about_denominators_assumed_nonzero = true;
 	evalops.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
+	evalops.parse_options.dot_as_separator = KGlobal::locale()->thousandsSeparator() == ".";
+
+	rpn_mode = false;
+	rpn_keypad_only = true;
 
 	save_mode_as(i18n("Preset"));
 	save_mode_as(i18n("Default"));
@@ -197,12 +206,13 @@ void load_preferences() {
 
 	status_warning_color.setNamedColor("blue");
 	status_error_color.setNamedColor("red");
-	use_icon_buttons = -1;
+	use_icon_buttons = 1;
 	enable_expression_completion = true;
 	enable_plot_expression_completion = true;
 	close_to_systray = false;
-	show_keypad = false;
+	show_keypad = true;
 	show_history = false;
+	show_stack = true;
 	display_expression_status = true;
 	save_mode_on_exit = true;
 	save_defs_on_exit = true;
@@ -227,7 +237,7 @@ void load_preferences() {
 		closedir(dir);
 	}
 
-	int version_numbers[] = {0, 9, 2};
+	int version_numbers[] = {0, 9, 4};
 	FILE *file = NULL;
 	filename += "qalculate-kde.cfg";
 	file = fopen(filename.c_str(), "r");
@@ -386,6 +396,11 @@ void load_preferences() {
 						else modes[mode_index].eo.approximation = (ApproximationMode) v;
 					}
 				} else if(svar == "in_rpn_mode") {
+					if(mode_index == 1) rpn_mode = v;
+					else modes[mode_index].rpn_mode = v;
+				} else if(svar == "rpn_keypad_only") {
+					rpn_keypad_only = v;
+				} else if(svar == "rpn_syntax") {
 					if(mode_index == 1) evalops.parse_options.rpn = v;
 					else modes[mode_index].eo.parse_options.rpn = v;
 				} else if(svar == "limit_implicit_multiplication") {
@@ -427,14 +442,18 @@ void load_preferences() {
 					printops.lower_case_numbers = v;
 				} else if(svar == "lower_case_e") {
 					printops.lower_case_e = v;
+				} else if(svar == "base_display") {
+					if(v >= BASE_DISPLAY_NONE && v <= BASE_DISPLAY_ALTERNATIVE) printops.base_display = (BaseDisplay) v;
 				} else if(svar == "spell_out_logical_operators") {
 					printops.spell_out_logical_operators = v;
+				} else if(svar == "dot_as_separator") {
+					evalops.parse_options.dot_as_separator = v;
 				} else if(svar == "use_custom_result_font") {
 					use_custom_result_font = v;
 				} else if(svar == "use_custom_expression_font") {
-					use_custom_expression_font = v;											
+					use_custom_expression_font = v;
 				} else if(svar == "use_custom_status_font") {
-					use_custom_status_font = v;											
+					use_custom_status_font = v;
 				} else if(svar == "custom_result_font") {
 					custom_result_font = svalue;
 				} else if(svar == "custom_expression_font") {
@@ -446,15 +465,17 @@ void load_preferences() {
 				} else if(svar == "status_warning_color") {
 					status_warning_color.setNamedColor(svalue);
 				} else if(svar == "multiplication_sign") {
-					printops.multiplication_sign = (MultiplicationSign) v;
+					if(v >= MULTIPLICATION_SIGN_ASTERISK && v <= MULTIPLICATION_SIGN_X) printops.multiplication_sign = (MultiplicationSign) v;
 				} else if(svar == "division_sign") {
-					printops.division_sign = (DivisionSign) v;
-				} else if(svar == "close_to_systray") {
+					if(v >= DIVISION_SIGN_SLASH && v <= DIVISION_SIGN_DIVISION) printops.division_sign = (DivisionSign) v;
+				} else if(svar == "close_to_system_tray") {
 					close_to_systray = v;
 				} else if(svar == "show_keypad") {
 					show_keypad = v;
 				} else if(svar == "show_history") {
 					show_history = v;
+				} else if(svar == "show_stack") {
+					show_stack = v;
 				} else if(svar == "display_expression_status") {
 					display_expression_status = v;
 				} else if(svar == "enable_expression_completion") {
@@ -466,7 +487,7 @@ void load_preferences() {
 				} else if(svar == "plot_style") {
 					if(v >= PLOT_STYLE_LINES && v <= PLOT_STYLE_DOTS) default_plot_style = (PlotStyle) v;
 				} else if(svar == "plot_smoothing") {
-					if(v >= PLOT_SMOOTHING_NONE && v <= PLOT_SMOOTHING_SBEZIER) default_plot_smoothing = (PlotSmoothing) v;		
+					if(v >= PLOT_SMOOTHING_NONE && v <= PLOT_SMOOTHING_SBEZIER) default_plot_smoothing = (PlotSmoothing) v;
 				} else if(svar == "plot_display_grid") {
 					default_plot_display_grid = v;
 				} else if(svar == "plot_full_border") {
@@ -578,7 +599,7 @@ void load_preferences() {
 	} else {
 		first_time = true;
 	}
-	
+	if(show_keypad) show_history = false;
 	//remember start mode for when we save preferences
 	set_saved_mode();
 	
@@ -601,16 +622,20 @@ void save_preferences(bool mode) {
 	fprintf(file, "save_definitions_on_exit=%i\n", save_defs_on_exit);
 	fprintf(file, "load_global_definitions=%i\n", load_global_defs);
 	fprintf(file, "fetch_exchange_rates_at_startup=%i\n", fetch_exchange_rates_at_startup);
-	fprintf(file, "close_to_systray=%i\n", close_to_systray);
-	fprintf(file, "show_keypad=%i\n", mainWin->keypadButton->isOn());
-	fprintf(file, "show_history=%i\n", mainWin->historyButton->isOn());
+	fprintf(file, "close_to_system_tray=%i\n", close_to_systray);
+	fprintf(file, "show_keypad=%i\n", mainWin->keypadButton->isOn() || (rpn_mode && show_keypad && mainWin->stackButton->isOn()));
+	fprintf(file, "show_history=%i\n", mainWin->historyButton->isOn() || (rpn_mode && show_history && mainWin->stackButton->isOn()));
+	fprintf(file, "show_stack=%i\n", rpn_mode ? mainWin->stackButton->isOn() : show_stack);
+	fprintf(file, "rpn_keypad_only=%i\n", rpn_keypad_only);
 	fprintf(file, "display_expression_status=%i\n", display_expression_status);
 	fprintf(file, "enable_expression_completion=%i\n", ((QalculateExpressionEdit*) expressionWidget)->completionEnabled());
 	if(use_icon_buttons >= 0) fprintf(file, "use_icon_buttons=%i\n", use_icon_buttons);
 	fprintf(file, "use_unicode_signs=%i\n", printops.use_unicode_signs);
 	fprintf(file, "lower_case_numbers=%i\n", printops.lower_case_numbers);
 	fprintf(file, "lower_case_e=%i\n", printops.lower_case_e);
+	fprintf(file, "base_display=%i\n", printops.base_display);
 	fprintf(file, "spell_out_logical_operators=%i\n", printops.spell_out_logical_operators);
+	fprintf(file, "dot_as_separator=%i\n", evalops.parse_options.dot_as_separator);
 	fprintf(file, "use_custom_result_font=%i\n", use_custom_result_font);	
 	fprintf(file, "use_custom_expression_font=%i\n", use_custom_expression_font);	
 	fprintf(file, "use_custom_status_font=%i\n", use_custom_status_font);	
@@ -688,7 +713,8 @@ void save_preferences(bool mode) {
 		fprintf(file, "show_ending_zeroes=%i\n", modes[i].po.show_ending_zeroes);
 		fprintf(file, "round_halfway_to_even=%i\n", modes[i].po.round_halfway_to_even);
 		fprintf(file, "approximation=%i\n", modes[i].eo.approximation);	
-		fprintf(file, "in_rpn_mode=%i\n", modes[i].eo.parse_options.rpn);
+		fprintf(file, "in_rpn_mode=%i\n", modes[i].rpn_mode);
+		fprintf(file, "rpn_syntax=%i\n", modes[i].eo.parse_options.rpn);
 		fprintf(file, "limit_implicit_multiplication=%i\n", modes[i].eo.parse_options.limit_implicit_multiplication);
 		fprintf(file, "spacious=%i\n", modes[i].po.spacious);
 		fprintf(file, "excessive_parenthesis=%i\n", modes[i].po.excessive_parenthesis);
