@@ -208,11 +208,15 @@ void QalculateEditDataSetDialog::slotOk() {
 	remove_blank_ends(str);
 	if(str.empty() && (!names_edit_dialog || names_edit_dialog->isEmpty())) {
 		//no name -- open dialog again
+		tabs->setCurrentPage(2);
+		nameEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty name field"));
 		return;
 	}
 	//function with the same name exists -- overwrite or open dialog again
 	if((!edited_dataset || !edited_dataset->hasName(str)) && (!names_edit_dialog || names_edit_dialog->isEmpty()) && CALCULATOR->functionNameTaken(str, edited_dataset) && KMessageBox::questionYesNo(this, i18n("A function with the same name already exists.\nDo you want to overwrite it?")) == KMessageBox::No) {
+		tabs->setCurrentPage(2);
+		nameEdit->setFocus();
 		return;
 	}
 	
@@ -260,17 +264,15 @@ void QalculateEditDataSetDialog::slotOk() {
 		}
 	}
 
-	if(!ds->isBuiltin()) {
-		if(names_edit_dialog) {
-			names_edit_dialog->saveNames(ds, str.c_str());
+	if(names_edit_dialog) {
+		names_edit_dialog->saveNames(ds, str.c_str());
+	} else {
+		if(ds->countNames() == 0) {
+			ExpressionName ename(str);
+			ename.reference = true;
+			ds->setName(ename, 1);
 		} else {
-			if(ds->countNames() == 0) {
-				ExpressionName ename(str);
-				ename.reference = true;
-				ds->setName(ename, 1);
-			} else {
-				ds->setName(str, 1);
-			}
+			ds->setName(str, 1);
 		}
 	}
 	if(add_func) {
@@ -468,6 +470,7 @@ QalculateEditDataPropertyDialog::QalculateEditDataPropertyDialog(QWidget *parent
 	grid->addWidget(typeCombo, 5, 1);
 	hideButton = new QCheckBox(i18n("Hide"), mainWidget());
 	grid->addWidget(hideButton, 6, 0);
+	//Use as a unique data set key
 	keyButton = new QCheckBox(i18n("Use as key"), mainWidget());
 	grid->addWidget(keyButton, 6, 1);
 	approximateButton = new QCheckBox(i18n("Approximate value"), mainWidget());
@@ -492,6 +495,7 @@ void QalculateEditDataPropertyDialog::slotOk() {
 	remove_blank_ends(str);
 	if(str.empty() && (!names_edit_dialog || names_edit_dialog->isEmpty())) {
 		//no name -- open dialog again
+		nameEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty name field"));
 		return;
 	}
