@@ -49,6 +49,7 @@
 #include <kstdguiitem.h>
 
 extern PrintOptions printops;
+extern EvaluationOptions evalops;
 extern tree_struct function_cats;
 
 QalculateEditFunctionDialog::QalculateEditFunctionDialog(QWidget *parent, const char *name) : KDialogBase(parent, name, true, i18n("Edit Function"), Ok | Cancel | Help, Ok, false) {
@@ -179,21 +180,27 @@ void QalculateEditFunctionDialog::slotOk() {
 
 	string str = nameEdit->text().ascii();
 	remove_blank_ends(str);
-	string str2 = CALCULATOR->unlocalizeExpression(expressionEdit->text().ascii());
+	string str2 = CALCULATOR->unlocalizeExpression(expressionEdit->text().ascii(), evalops.parse_options);
 	remove_blank_ends(str2);
 	gsub("\n", " ", str2);
 	if(str.empty() && (!names_edit_dialog || names_edit_dialog->isEmpty())) {
 		//no name -- open dialog again
+		tabs->setCurrentPage(0);
+		nameEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty name field"));
 		return;
 	}
 	if(!(edited_function && edited_function->isBuiltin()) && str2.empty()) {
-		//no expression/relation -- open dialog again
+		//no expression/relation -- open dialog again		
+		tabs->setCurrentPage(1);
+		expressionEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty expression field"));
 		return;
 	}
 	//function with the same name exists -- overwrite or open dialog again
 	if((!edited_function || !edited_function->hasName(str)) && (!names_edit_dialog || names_edit_dialog->isEmpty()) && CALCULATOR->functionNameTaken(str, edited_function) && KMessageBox::questionYesNo(this, i18n("A function with the same name already exists.\nDo you want to overwrite it?")) == KMessageBox::No) {
+		tabs->setCurrentPage(0);
+		nameEdit->setFocus();
 		return;
 	}
 	
@@ -206,15 +213,12 @@ void QalculateEditFunctionDialog::slotOk() {
 		f->setCategory(categoryCombo->lineEdit()->text().ascii());
 		f->setTitle(titleEdit->text().ascii());
 		f->setDescription(descriptionEdit->text().ascii());
-		if(!f->isBuiltin()) {
-			((UserFunction*) f)->setEquation(str2);
-		}
 	} else {
 		//new function
-		f = new UserFunction(categoryCombo->lineEdit()->text().ascii(), "", str2, true, -1, titleEdit->text().ascii(), descriptionEdit->text().ascii());
+		f = new UserFunction(categoryCombo->lineEdit()->text().ascii(), "", "", true, -1, titleEdit->text().ascii(), descriptionEdit->text().ascii());
 		add_func = true;
 	}
-	f->setCondition(CALCULATOR->unlocalizeExpression(conditionEdit->text().ascii()));
+	f->setCondition(CALCULATOR->unlocalizeExpression(conditionEdit->text().ascii(), evalops.parse_options));
 	size_t index = 1;
 	QListViewItemIterator it(argumentsView);
 	while(it.current()) {
@@ -252,6 +256,7 @@ void QalculateEditFunctionDialog::slotOk() {
 		if(subfunctions_edit_dialog) {
 			subfunctions_edit_dialog->saveSubFunctions((UserFunction*) f);
 		}
+		((UserFunction*) f)->setFormula(str2);
 	}
 	if(add_func) {
 		CALCULATOR->addFunction(f);
@@ -338,7 +343,7 @@ MathFunction *QalculateEditFunctionDialog::editFunction(QString category, MathFu
 		//fill in original paramaters
 		set_name_label_and_entry(f, nameEdit, namesLabel);
 		if(!f->isBuiltin()) {
-			expressionEdit->setText(CALCULATOR->localizeExpression(((UserFunction*) f)->equation()).c_str());
+			expressionEdit->setText(CALCULATOR->localizeExpression(((UserFunction*) f)->formula()).c_str());
 		}
 		categoryCombo->lineEdit()->setText(f->category().c_str());
 		titleEdit->setText(f->title(false).c_str());
@@ -726,6 +731,10 @@ Argument *QalculateEditArgumentDialog::editArgument(Argument *arg) {
 		matrixButton->setChecked(arg->matrixAllowed());
 		zeroButton->setChecked(arg->zeroForbidden());
 		testButton->setChecked(arg->tests());
+		if(!arg->getCustomCondition().empty()) {
+			conditionButton->setChecked(true);
+			conditionEdit->setText(arg->getCustomCondition().c_str());
+		}
 	}
 	typeChanged(typeCombo->currentItem());
 
@@ -774,6 +783,11 @@ Argument *QalculateEditArgumentDialog::editArgument(Argument *arg) {
 		arg->setName(nameEdit->text().ascii());
 		arg->setZeroForbidden(zeroButton->isChecked());
 		arg->setTests(testButton->isChecked());
+		if(conditionButton->isChecked()) {
+			arg->setCustomCondition(conditionEdit->text().ascii());
+		} else {
+			arg->setCustomCondition("");
+		}
 		if(arg->type() != ARGUMENT_TYPE_FREE && arg->type() != ARGUMENT_TYPE_MATRIX) arg->setMatrixAllowed(matrixButton->isChecked());
 		return arg;	
 	}
