@@ -40,6 +40,7 @@ extern vector<mode_struct> modes;
 extern KQalculate *mainWin;
 extern PrintOptions printops;
 extern EvaluationOptions evalops;
+extern bool rpn_mode, rpn_keypad_only;
 extern tree_struct function_cats, unit_cats, variable_cats;
 extern vector<void*> ia_units, ia_variables, ia_functions;
 
@@ -385,21 +386,47 @@ void QalculateExpressionEdit::wrapSelection() {
 }
 
 void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
+	if(e->key() == Key_Enter || e->key() == Key_Return) {
+		mainWin->execute();
+		e->accept();
+		return;
+	}
+	if(e->state() & ControlButton && e->key() == Key_Asterisk) {
+		if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+			mainWin->calculateRPN(OPERATION_RAISE);
+			return;
+		}
+		insert("^");
+		e->accept();
+		return;
+	}
 	if(e->state() != 0 && e->state() != ShiftButton && e->state() != Keypad) {
 		KLineEdit::keyPressEvent(e);
 		return;
 	}
 	switch(e->key()) {
 		case Key_AsciiCircum: {
-			wrapSelection();
-			deselect();
+			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+				mainWin->calculateRPN(OPERATION_RAISE);
+				return;
+			}
+			if(!evalops.parse_options.rpn) {
+				wrapSelection();
+				deselect();
+			}
 			insert("^");
 			e->accept();
 			return;
 		}
 		case Key_Slash: {
-			wrapSelection();
-			deselect();
+			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+				mainWin->calculateRPN(OPERATION_DIVIDE);
+				return;
+			}
+			if(!evalops.parse_options.rpn) {
+				wrapSelection();
+				deselect();
+			}
 			if(printops.use_unicode_signs && printops.division_sign == DIVISION_SIGN_DIVISION && can_display_unicode_string_function(SIGN_DIVISION, (void*) this)) {
 				insert(SIGN_DIVISION);
 				e->accept();
@@ -408,8 +435,14 @@ void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
 			break;
 		}
 		case Key_Asterisk: {
-			wrapSelection();
-			deselect();
+			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+				mainWin->calculateRPN(OPERATION_MULTIPLY);
+				return;
+			}
+			if(!evalops.parse_options.rpn) {
+				wrapSelection();
+				deselect();
+			}
 			if(printops.use_unicode_signs && printops.multiplication_sign == MULTIPLICATION_SIGN_DOT && can_display_unicode_string_function(SIGN_MULTIDOT, (void*) this)) {
 				insert(SIGN_MULTIDOT);
 			} else if(printops.use_unicode_signs && printops.multiplication_sign == MULTIPLICATION_SIGN_DOT && can_display_unicode_string_function(SIGN_SMALLCIRCLE, (void*) this)) {
@@ -423,14 +456,26 @@ void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
 			return;
 		}
 		case Key_Plus: {
-			wrapSelection();
-			deselect();
+			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+				mainWin->calculateRPN(OPERATION_ADD);
+				return;
+			}
+			if(!evalops.parse_options.rpn) {
+				wrapSelection();
+				deselect();
+			}
 			e->accept();
 			break;
 		}
 		case Key_Minus: {
-			wrapSelection();
-			deselect();
+			if(rpn_mode && b_main && (!rpn_keypad_only || e->state() & Keypad)) {
+				mainWin->calculateRPN(OPERATION_SUBTRACT);
+				return;
+			}
+			if(!evalops.parse_options.rpn) {
+				wrapSelection();
+				deselect();
+			}
 			if(printops.use_unicode_signs && can_display_unicode_string_function(SIGN_MINUS, (void*) this)) {
 				insert(SIGN_MINUS);
 				e->accept();
@@ -475,64 +520,82 @@ void QalculateExpressionEdit::keyPressEvent(QKeyEvent *e) {
 				return;
 			}
 			if(e->text().utf8() == "¹") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^1");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "²") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^2");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "³") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^3");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁴") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^4");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁵") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^5");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁶") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^6");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁷") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^7");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁸") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^8");
 				e->accept();
 				return;
 			}
 			if(e->text().utf8() == "⁹") {
-				wrapSelection();
-				deselect();
+				if(!evalops.parse_options.rpn) {
+					wrapSelection();
+					deselect();
+				}
 				insert("^9");
 				e->accept();
 				return;
