@@ -115,7 +115,7 @@ QalculatePlotDialog::QalculatePlotDialog(QWidget *parent, const char *name) : KD
 	styleCombo->insertItem(i18n("Line"));
 	styleCombo->insertItem(i18n("Points"));
 	styleCombo->insertItem(i18n("Line with points"));
-	styleCombo->insertItem(i18n("Boxes/Bars"));
+	styleCombo->insertItem(i18n("Boxes/bars"));
 	styleCombo->insertItem(i18n("Histogram"));
 	styleCombo->insertItem(i18n("Steps"));
 	styleCombo->insertItem(i18n("Candlesticks"));
@@ -224,10 +224,10 @@ QalculatePlotDialog::QalculatePlotDialog(QWidget *parent, const char *name) : KD
 	grid->addWidget(new QLabel(i18n("Legend placement:"), page3), 7, 0);
 	legendPlacementCombo = new KComboBox(page3);
 	legendPlacementCombo->insertItem(i18n("Hide"));
-	legendPlacementCombo->insertItem(i18n("Top-Left"));
-	legendPlacementCombo->insertItem(i18n("Top-Right"));
-	legendPlacementCombo->insertItem(i18n("Bottom-Left"));
-	legendPlacementCombo->insertItem(i18n("Bottom-Right"));
+	legendPlacementCombo->insertItem(i18n("Top-left"));
+	legendPlacementCombo->insertItem(i18n("Top-right"));
+	legendPlacementCombo->insertItem(i18n("Bottom-left"));
+	legendPlacementCombo->insertItem(i18n("Bottom-right"));
 	legendPlacementCombo->insertItem(i18n("Below"));
 	legendPlacementCombo->insertItem(i18n("Outside"));
 	grid->addWidget(legendPlacementCombo, 7, 1);
@@ -454,6 +454,7 @@ void QalculatePlotDialog::expressionActivated() {
 void QalculatePlotDialog::addSeries() {
 	QString expression = expressionEdit->text();
 	if(expression.stripWhiteSpace().isEmpty()) {
+		expressionEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty expression."));
 		return;
 	}
@@ -467,6 +468,7 @@ void QalculatePlotDialog::addSeries() {
 	}
 	QString str_x = xVariableEdit->text().stripWhiteSpace();
 	if(str_x.isEmpty() && type == 0) {
+		xVariableEdit->setFocus();
 		KMessageBox::error(this, i18n("Empty x variable."));
 		return;
 	}
@@ -500,6 +502,7 @@ void QalculatePlotDialog::modifySeries() {
 	if(i) {
 		QString expression = expressionEdit->text();
 		if(expression.stripWhiteSpace().isEmpty()) {
+			expressionEdit->setFocus();
 			KMessageBox::error(this, i18n("Empty expression."));
 			return;
 		}
@@ -512,6 +515,7 @@ void QalculatePlotDialog::modifySeries() {
 		}
 		QString str_x = xVariableEdit->text().stripWhiteSpace();
 		if(str_x.isEmpty() && type == 0) {
+			xVariableEdit->setFocus();
 			KMessageBox::error(this, i18n("Empty x variable."));
 			return;
 		}
@@ -583,7 +587,7 @@ void QalculatePlotDialog::applyAppearance() {
 	updatePlot();
 }
 
-bool QalculatePlotDialog::generatePlot(plot_parameters &pp, vector<MathStructure> &y_vectors, vector<MathStructure> &x_vectors, vector<plot_data_parameters*> &pdps) {
+bool QalculatePlotDialog::generatePlot(PlotParameters &pp, vector<MathStructure> &y_vectors, vector<MathStructure> &x_vectors, vector<PlotDataParameters*> &pdps) {
 	QListViewItemIterator it(seriesView);
 	if(!it.current()) {
 		return false;
@@ -657,7 +661,7 @@ bool QalculatePlotDialog::generatePlot(plot_parameters &pp, vector<MathStructure
 			x_vectors.push_back(*x_vector);
 		}
 		for(int i = 0; i < count; i++) {
-			plot_data_parameters *pdp = new plot_data_parameters();
+			PlotDataParameters *pdp = new PlotDataParameters();
 			pdp->title = title.ascii();
 			if(count > 1) {
 				pdp->title += " :";
@@ -734,8 +738,8 @@ void QalculatePlotDialog::savePlot() {
 	}
 	vector<MathStructure> y_vectors;
 	vector<MathStructure> x_vectors;
-	vector<plot_data_parameters*> pdps;
-	plot_parameters pp;
+	vector<PlotDataParameters*> pdps;
+	PlotParameters pp;
 	if(generatePlot(pp, y_vectors, x_vectors, pdps)) {
 		pp.filename = filename.ascii();
 		pp.filetype = PLOT_FILETYPE_AUTO;
@@ -748,8 +752,8 @@ void QalculatePlotDialog::savePlot() {
 void QalculatePlotDialog::updatePlot() {
 	vector<MathStructure> y_vectors;
 	vector<MathStructure> x_vectors;
-	vector<plot_data_parameters*> pdps;
-	plot_parameters pp;
+	vector<PlotDataParameters*> pdps;
+	PlotParameters pp;
 	if(!generatePlot(pp, y_vectors, x_vectors, pdps)) {
 		CALCULATOR->closeGnuplot();
 		actionButton(User1)->setEnabled(false);
@@ -765,16 +769,17 @@ void QalculatePlotDialog::generatePlotSeries(MathStructure **x_vector, MathStruc
 	EvaluationOptions eo;
 	eo.approximation = APPROXIMATION_APPROXIMATE;
 	eo.parse_options = evalops.parse_options;
+	eo.parse_options.read_precision = DONT_READ_PRECISION;
 	if(type == 1 || type == 2) {
-		*y_vector = new MathStructure(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(str), eo));
+		*y_vector = new MathStructure(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(str, evalops.parse_options), eo));
 		*x_vector = NULL;
 	} else {
-		MathStructure min(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(minEdit->text().ascii()), eo));
-		MathStructure max(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(maxEdit->text().ascii()), eo));
+		MathStructure min(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(minEdit->text().ascii(), evalops.parse_options), eo));
+		MathStructure max(CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(maxEdit->text().ascii(), evalops.parse_options), eo));
 		*x_vector = new MathStructure();
 		(*x_vector)->clearVector();
 		if(stepSizeButton->isChecked()) {
-			*y_vector = new MathStructure(CALCULATOR->expressionToPlotVector(str, min, max, CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(stepSizeEdit->text().ascii()), eo), *x_vector, str_x, evalops.parse_options));
+			*y_vector = new MathStructure(CALCULATOR->expressionToPlotVector(str, min, max, CALCULATOR->calculate(CALCULATOR->unlocalizeExpression(stepSizeEdit->text().ascii(), evalops.parse_options), eo), *x_vector, str_x, evalops.parse_options));
 		} else {
 			*y_vector = new MathStructure(CALCULATOR->expressionToPlotVector(str, min, max, samplingRateBox->value(), *x_vector, str_x, evalops.parse_options));
 		}
