@@ -382,9 +382,9 @@ void QalculateUnitsDialog::addUnitTreeItem(Unit *u) {
 		case SUBTYPE_ALIAS_UNIT: {
 			au = (AliasUnit*) u;
 			sbase = au->firstBaseUnit()->preferredDisplayName(printops.abbreviate_names, printops.use_unicode_signs, false, false, &can_display_unicode_string_function, (void*) unitView).name.c_str();
-			if(au->firstBaseExp() != 1) {
+			if(au->firstBaseExponent() != 1) {
 				sbase += POWER;
-				sbase += QString::number(au->firstBaseExp());
+				sbase += QString::number(au->firstBaseExponent());
 			}
 			break;
 		}
@@ -512,7 +512,7 @@ void QalculateUnitsDialog::convert() {
 			PrintOptions po;
 			po.is_approximate = &b;
 			po.number_fraction_format = FRACTION_DECIMAL;
-			MathStructure v_mstruct = CALCULATOR->convert(fromEdit->text().ascii(), selected_unit, selected_to_unit, eo);
+			MathStructure v_mstruct = CALCULATOR->convert(CALCULATOR->unlocalizeExpression(fromEdit->text().ascii(), evalops.parse_options), selected_unit, selected_to_unit, eo);
 			toEdit->setText(v_mstruct.print(po).c_str());
 			b = b || v_mstruct.isApproximate();
 		}
@@ -535,7 +535,7 @@ void QalculateUnitsDialog::convertReversed() {
 			PrintOptions po;
 			po.is_approximate = &b;
 			po.number_fraction_format = FRACTION_DECIMAL;
-			MathStructure v_mstruct = CALCULATOR->convert(toEdit->text().ascii(), selected_to_unit, selected_unit, eo);
+			MathStructure v_mstruct = CALCULATOR->convert(CALCULATOR->unlocalizeExpression(toEdit->text().ascii(), evalops.parse_options), selected_to_unit, selected_unit, eo);
 			fromEdit->setText(v_mstruct.print(po).c_str());
 			b = b || v_mstruct.isApproximate();
 		}
